@@ -12,66 +12,21 @@ tests under TeamCity build.
 """
 
 import os
-import pprint
-import sys
 import re
+import sys
 import traceback
 from datetime import timedelta
 
-from teamcity.messages import TeamcityServiceMessages
-from teamcity.common import convert_error_to_string, dump_test_stderr, dump_test_stdout
-from teamcity import is_running_under_teamcity
+import pytest
+
 from teamcity import diff_tools
+from teamcity import is_running_under_teamcity
+from teamcity.common import convert_error_to_string, dump_test_stderr, dump_test_stdout
+from teamcity.messages import TeamcityServiceMessages
+from teamcity.output import TeamCityMessagesPrinter
 
 diff_tools.patch_unittest_diff()
-
-
-def unformat_pytest_explanation(s):
-    """
-    Undo _pytest.assertion.util.format_explanation
-    """
-    return s.replace("\\n", "\n")
-
-
-def fetch_diff_error_from_message(err_message, swap_diff):
-    line_with_diff = None
-    diff_error_message = None
-    lines = err_message.split("\n")
-    if err_message.startswith("AssertionError: assert"):
-        # Everything in one line
-        line_with_diff = lines[0][len("AssertionError: assert "):]
-    elif len(err_message.split("\n")) > 1:
-        err_line = lines[1]
-        line_with_diff = err_line[len("assert "):]
-        diff_error_message = lines[0]
-
-    if line_with_diff and line_with_diff.count("==") == 1:
-        parts = [x.strip() for x in line_with_diff.split("==")]
-        parts = [s[1:-1] if s.startswith("'") or s.startswith('"') else s for s in parts]
-        # Pytest cuts too long lines, no need to check is_too_big
-        expected, actual = parts[1], parts[0]
-
-        if swap_diff:
-            expected, actual = actual, expected
-
-        expected = unformat_pytest_explanation(expected)
-        actual = unformat_pytest_explanation(actual)
-
-        return diff_tools.EqualsAssertionError(expected, actual, diff_error_message)
-    else:
-        return None
-
-
-def _is_bool_supported():
-    """
-    Type "bool" is not supported before 2.9
-    """
-    try:
-        from pytest import __version__
-        from distutils import version
-        return version.LooseVersion(str(__version__)) >= version.LooseVersion("2.9")
-    except ImportError:
-        return False
+_ASSERTION_FAILURE_KEY = '_teamcity_assertion_failure'
 
 
 def pytest_addoption(parser):
@@ -84,13 +39,19 @@ def pytest_addoption(parser):
     parser.addoption('--jb-swapdiff', action="store_true", dest="swapdiff", default=False, help="Swap actual/expected in diff")
 
     kwargs = {"help": "skip output of passed tests for JetBrains TeamCity service messages"}
-    if _is_bool_supported():
-        kwargs.update({"type": "bool"})
+    kwargs.update({"type": "bool"})
 
     parser.addini("skippassedoutput", **kwargs)
     parser.addini("swapdiff", **kwargs)
 
 
+def get_rootdir(config):
+    if hasattr(config, 'rootpath'):  # pytest>=6
+        return str(config.rootpath)
+    else:
+        return str(config.rootdir)
+
+
 def pytest_configure(config):
     if config.option.no_teamcity >= 1:
         enabled = False
@@ -107,9 +68,12 @@ def pytest_configure(config):
         config.option.verbose = 2  # don't truncate assert explanations
         config._teamcityReporting = EchoTeamCityMessages(
             output_capture_enabled,
+            # never write tc messages into buffered output
+            getattr(config.pluginmanager.getplugin('capturemanager'), 'global_and_fixture_disabled'),
             coverage_controller,
             skip_passed_output,
-            bool(config.getini('swapdiff') or config.option.swapdiff)
+            bool(config.getini('swapdiff') or config.option.swapdiff),
+            get_rootdir(config),
         )
         config.pluginmanager.register(config._teamcityReporting)
 
@@ -130,13 +94,16 @@ def _get_coverage_controller(config):
 
 
 class EchoTeamCityMessages(object):
-    def __init__(self, output_capture_enabled, coverage_controller, skip_passed_output, swap_diff):
+    def __init__(self, output_capture_enabled, context_manager, coverage_controller, skip_passed_output, swap_diff, rootdir):
         self.coverage_controller = coverage_controller
         self.output_capture_enabled = output_capture_enabled
         self.skip_passed_output = skip_passed_output
 
-        self.teamcity = TeamcityServiceMessages()
+        output_handler = TeamCityMessagesPrinter(context_manager=context_manager)
+        self.teamcity = TeamcityServiceMessages(output_handler=output_handler)
         self.test_start_reported_mark = set()
+        self.rootdir = rootdir
+        self.current_test_item = None
 
         self.max_reported_output_size = 1 * 1024 * 1024
         self.reported_output_chunk_size = 50000
@@ -209,6 +176,16 @@ class EchoTeamCityMessages(object):
             return "%s:%s (%s)" % (str(location[0]), str(location[1]), str(location[2]))
         return str(location)
 
+    def pytest_sessionfinish(self, session, exitstatus):
+        if exitstatus > pytest.ExitCode.TESTS_FAILED and self.current_test_item:
+            test_id = self.format_test_id(self.current_test_item.nodeid, self.current_test_item.location)
+            self.teamcity.testStopped(
+                test_id,
+                message=exitstatus.name if hasattr(exitstatus, 'name') else str(exitstatus),
+                flowId=test_id
+            )
+            self.report_test_finished(test_id)
+
     def pytest_collection_finish(self, session):
         self.teamcity.testCount(len(session.items))
 
@@ -216,18 +193,25 @@ class EchoTeamCityMessages(object):
         # test name fetched from location passed as metainfo to PyCharm
         # it will be used to run specific test
         # See IDEA-176950, PY-31836
-        test_name = location[2]
+        path, lineno, test_name = location
         if test_name:
             test_name = str(test_name).split(".")[-1]
-        self.ensure_test_start_reported(self.format_test_id(nodeid, location), test_name)
+        path = os.path.join(self.rootdir, path)
+        lineno = lineno + 1 if lineno is not None else None
+        self.ensure_test_start_reported(self.format_test_id(nodeid, location), test_name, path=path, lineno=lineno)
 
-    def ensure_test_start_reported(self, test_id, metainfo=None):
+    def pytest_runtest_protocol(self, item):
+        self.current_test_item = item
+        return None  # continue to next hook
+
+    def ensure_test_start_reported(self, test_id, metainfo=None, path=None, lineno=None):
         if test_id not in self.test_start_reported_mark:
             if self.output_capture_enabled:
                 capture_standard_output = "false"
             else:
                 capture_standard_output = "true"
-            self.teamcity.testStarted(test_id, flowId=test_id, captureStandardOutput=capture_standard_output, metainfo=metainfo)
+            self.teamcity.testStarted(test_id, flowId=test_id, captureStandardOutput=capture_standard_output,
+                                      metainfo=metainfo, path=path, lineno=str(lineno) if lineno is not None else None)
             self.test_start_reported_mark.add(test_id)
 
     def report_has_output(self, report):
@@ -277,13 +261,18 @@ class EchoTeamCityMessages(object):
                 serialized_data = err_message[err_message.index(diff_name) + len(diff_name) + 1:]
                 diff_error = diff_tools.deserialize_error(serialized_data)
 
-            # AssertionError is patched in py.test, we can try to fetch diff from it
-            # In general case message starts with "AssertionError: ", but can also starts with "assert" for top-level
-            # function. To support both cases we unify them
-            if err_message.startswith("assert"):
-                err_message = "AssertionError: " + err_message
-            if err_message.startswith("AssertionError:"):
-                diff_error = fetch_diff_error_from_message(err_message, self.swap_diff)
+            assertion_tuple = getattr(self.current_test_item, _ASSERTION_FAILURE_KEY, None)
+            if assertion_tuple:
+                op, left, right = assertion_tuple
+                if self.swap_diff:
+                    left, right = right, left
+                diff_error = diff_tools.EqualsAssertionError(expected=right, actual=left)
+            else:
+                if m := re.search("AssertionError: Expected <(.*)> to .*? <(.*)>, but .*", err_message):
+                    left, right = m.group(1), m.group(2)
+                    if self.swap_diff:
+                        left, right = right, left
+                    diff_error = diff_tools.EqualsAssertionError(expected=right, actual=left)
         except Exception:
             pass
 
@@ -302,7 +291,7 @@ class EchoTeamCityMessages(object):
                 strace = strace[0:strace.index(data_postfix)].strip()
                 if strace.endswith(":") and diff_error.real_exception:
                     strace += " " + type(diff_error.real_exception).__name__
-            self.teamcity.testFailed(test_id, diff_error.msg if diff_error.msg else message, strace,
+            self.teamcity.testFailed(test_id, diff_error.msg or message, strace,
                                      flowId=test_id,
                                      comparison_failure=diff_error
                                      )
@@ -327,8 +316,7 @@ class EchoTeamCityMessages(object):
         self.report_test_finished(test_id, duration)
 
     def pytest_assertrepr_compare(self, config, op, left, right):
-        if op in ('==', '!='):
-            return ['{0} {1} {2}'.format(pprint.pformat(left), op, pprint.pformat(right))]
+        setattr(self.current_test_item, _ASSERTION_FAILURE_KEY, (op, left, right))
 
     def pytest_runtest_logreport(self, report):
         """
@@ -438,7 +426,7 @@ class EchoTeamCityMessages(object):
 
                 for cu in units:
                     try:
-                        analysis = self.coverage._analyze(cu)
+                        analysis = self.coverage._analyze(cu.filename)
                         nums = analysis.numbers
                         total += nums
                     except KeyboardInterrupt:
@@ -452,7 +440,7 @@ class EchoTeamCityMessages(object):
                         if typ is NotPython and not cu.should_be_python():
                             continue
 
-                        test_id = cu.name
+                        test_id = cu.relname
                         details = convert_error_to_string(err)
 
                         self.messages.testStarted(test_id, flowId=test_id)
