@@ -589,17 +589,26 @@ def array_to_xml(array, name, roffset, coffset, rows, cols, format):
     return xml
 
 
-def tensor_to_xml(tensor, name, roffset, coffset, rows, cols, format):
+def tf_to_xml(tensor, name, roffset, coffset, rows, cols, format):
     try:
         return array_to_xml(tensor.numpy(), name, roffset, coffset, rows, cols, format)
     except TypeError:
         return array_to_xml(tensor.to_dense().numpy(), name, roffset, coffset, rows, cols, format)
 
 
-def sparse_tensor_to_xml(tensor, name, roffset, coffset, rows, cols, format):
+def torch_to_xml(tensor, name, roffset, coffset, rows, cols, format):
+    try:
+        if tensor.requires_grad:
+            tensor = tensor.detach()
+        return array_to_xml(tensor.numpy(), name, roffset, coffset, rows, cols, format)
+    except TypeError:
+        return array_to_xml(tensor.to_dense().numpy(), name, roffset, coffset, rows, cols, format)
+
+
+def tf_sparse_to_xml(tensor, name, roffset, coffset, rows, cols, format):
     try:
         import tensorflow as tf
-        return tensor_to_xml(tf.sparse.to_dense(tf.sparse.reorder(tensor)), name, roffset, coffset, rows, cols, format)
+        return tf_to_xml(tf.sparse.to_dense(tf.sparse.reorder(tensor)), name, roffset, coffset, rows, cols, format)
     except ImportError:
         pass
 
@@ -861,10 +870,10 @@ TYPE_TO_XML_CONVERTERS = {
     "Series": dataframe_to_xml,
     "GeoDataFrame": dataframe_to_xml,
     "GeoSeries": dataframe_to_xml,
-    "EagerTensor": tensor_to_xml,
-    "ResourceVariable": tensor_to_xml,
-    "SparseTensor": sparse_tensor_to_xml,
-    "Tensor": tensor_to_xml,
+    "EagerTensor": tf_to_xml,
+    "ResourceVariable": tf_to_xml,
+    "SparseTensor": tf_sparse_to_xml,
+    "Tensor": torch_to_xml,
     "Dataset": dataset_to_xml
 }
 
